#include "stdafx.h"
#include "TipInstallUpdateCommand.h"
#include "CustomProgressBar.h"

namespace tipcommon
{
	CALLBACK_RETURN TipInstallUpdateCallback::OnPrepareStart( const TCHAR* guid )
	{
		ATLASSERT(m_Document);
		TipServiceData* service = m_Document->SearchService(guid);
		if (!service)
			return CALLBACK_RETURN_ERROR;

		ATLTRACE(_T("OnPrepareStart\n"));

		service->SetInstallStatus(InstallStatus::DOWNLOAD);

		return CALLBACK_RETURN_CONTINUE;
	}
	CALLBACK_RETURN TipInstallUpdateCallback::OnProgress( const TCHAR* guid, 
		CALLBACK_PROGRESS_SIZE currentSize,
		CALLBACK_PROGRESS_SIZE maxSize,
		CALLBACK_PROGRESS_STATUS status )
	{
		ATLASSERT(m_Document);
		TipServiceData* service = m_Document->SearchService(guid);
		if (!service)
			return CALLBACK_RETURN_CONTINUE;

		if (status == CALLBACK_PROGRESS_PENDING)
			return CALLBACK_RETURN_CONTINUE;

		//ATLTRACE(_T("%s OnProgress\n"), service->GetConstModel()->m_Name);
		double maxlen = maxSize;
		double curlen = currentSize;
		double pos = curlen/maxlen;

		service->SetInstallProgressPos(pos);

		CString str;
		str.FormatMessage(_T("%1!s! OnProgress\n"), service->GetName());
		ATLTRACE(str);

		return CALLBACK_RETURN_CONTINUE;
	}
	CALLBACK_RETURN TipInstallUpdateCallback::OnPrepareFinished( const TCHAR* guid,
		CALLBACK_FINISHED_STATUS status , ERRORINFO *errorInfo )
	{
		ATLASSERT(m_Document);
		TipServiceData* service = m_Document->SearchService(guid);
		if (!service)
			return CALLBACK_RETURN_CONTINUE;

		ATLTRACE(_T("OnPrepareFinished\n"));
		//ATLTRACE(_T("%s OnPrepareFinished\n"), service->GetConstModel()->m_Name);

		double strollValue = 0;
		CALLBACK_RETURN ret = CALLBACK_RETURN_CONTINUE;
		switch(status)
		{
		case CALLBACK_FINISHED_SUCCESS:
			service->SetInstallStatus(InstallStatus::READY);
			strollValue = TipProgressBar::MAX_VALUE;
			ret = CALLBACK_RETURN_CONTINUE;
			break;
		case CALLBACK_FINISHED_CANCEL:
			service->SetInstallStatus(InstallStatus::CANCEL);
			strollValue = TipProgressBar::CANCEL_VALUE;
			ret = CALLBACK_RETURN_CANCEL;
			break;
		case CALLBACK_FINISHED_ERROR:
			service->SetInstallStatus(InstallStatus::FAIL);
			strollValue = 0;
			ret = CALLBACK_RETURN_ERROR;
			break;
		}
		service->SetInstallProgressPos(strollValue);

		return ret;
	}
	CALLBACK_RETURN TipInstallUpdateCallback::OnInstallStart( const TCHAR* guid )
	{
		ATLASSERT(m_Document);
		TipServiceData* service = m_Document->SearchService(guid);
		if (!service)
			return CALLBACK_RETURN_CONTINUE;

		// ̹ ó ̸ ġ ʴ´.
		if( service->GetInstallStatus() == InstallStatus::CANCEL )
		{
			ATLTRACE(_T("InstallStatus::CANCEL CALLBACK_RETURN_CONTINUE\n"));
			return CALLBACK_RETURN_CANCEL;
		}

		ATLTRACE(_T("OnInstallStart\n"));

		if (service->GetInstallInfo() == InstallInfo::NOTINSTALLED)
			service->SetInstallStatus(InstallStatus::INSTALL);
		else
			service->SetInstallStatus(InstallStatus::UPDATE);

		service->SetInstallProgressPos(TipProgressBar::STROLL_VALUE);

		return CALLBACK_RETURN_CONTINUE;
	}
	CALLBACK_RETURN TipInstallUpdateCallback::OnInstallFinished( const TCHAR* guid,
		CALLBACK_FINISHED_STATUS status, ERRORINFO *errorInfo )
	{
		ATLASSERT(m_Document);
		TipServiceData* service = m_Document->SearchService(guid);
		if (!service)
			return CALLBACK_RETURN_CONTINUE;

		ATLTRACE(_T("OnInstallFinished\n"));
		//ATLTRACE(_T("%s OnInstallFinished\n"), service->GetConstModel()->m_Name);

		CALLBACK_RETURN ret = CALLBACK_RETURN_CONTINUE;
		switch(status)
		{
		case CALLBACK_FINISHED_SUCCESS:
			service->SetInstallStatus(InstallStatus::FINISH);
			service->SetInstallProgressPos(TipProgressBar::MAX_VALUE);
			ret = CALLBACK_RETURN_CONTINUE;
			break;
		case CALLBACK_FINISHED_CANCEL:
			service->SetInstallStatus(InstallStatus::CANCEL);
			// ġ Ҵ Progress 0 .
			service->SetInstallProgressPos(TipProgressBar::MIN_VALUE/*TipProgressBar::CANCEL_VALUE*/);
			ret = CALLBACK_RETURN_CANCEL;
			break;
		case CALLBACK_FINISHED_ERROR:
			service->SetInstallStatus(InstallStatus::FAIL);
			service->SetInstallProgressPos(TipProgressBar::MIN_VALUE);
			ret = CALLBACK_RETURN_ERROR;
			break;
		}

		return ret;
	}

};